#include "opencv2/opencv.hpp"

int main() {
    cv::namedWindow("input", cv::WINDOW_AUTOSIZE);
    cv::namedWindow("output", cv::WINDOW_AUTOSIZE);
    cv::VideoCapture cap(0);
    cv::Mat input;
    cap >> input;
    cv::Mat output = cv::Mat::zeros(input.rows, input.cols, CV_32F);
    cv::Mat Ix(input.rows, input.cols, CV_16S);
    cv::Mat Iy(input.rows, input.cols, CV_16S);

    while (1) {
        cap >> input;
        cv::cvtColor(input, input, CV_BGR2GRAY);

        for (int j = 1; j < input.rows - 1; j++) {
            for (int i = 1; i < input.cols - 1; i++) {
                Ix.at<short>(j, i) = 
                    (input.at<uchar>(j, i + 1) - input.at<uchar>(j, i - 1)) / 2;
                Iy.at<short>(j, i) = 
                    (input.at<uchar>(j + 1, i) - input.at<uchar>(j - 1, i)) / 2;
            }
        }

        float lambda_max = 0.0f;
        for (int j = 2; j < input.rows - 2; j++) {
            for (int i = 2; i < input.cols - 2; i++) {
                float g11 = 0.0f, g12 = 0.0f, g22 = 0.0f;
                for (int n = -1; n <= 1; n++) {
                    for (int m = -1; m <= 1; m++) {
                        float Ix_ = Ix.at<short>(j + n, i + m);
                        float Iy_ = Iy.at<short>(j + n, i + m);
                        g11 += Ix_ * Ix_;
                        g12 += Ix_ * Iy_;
                        g22 += Iy_ * Iy_;
                    }
                }
                float lambda2 =
                    (g11 + g22)
                    - sqrt((g11 - g22) * (g11 - g22) + 4 * g12 * g12);
                if (lambda2 > lambda_max) {
                    lambda_max = lambda2;
                }
                output.at<float>(j, i) = lambda2;
            }
        }

        for (int j = 2; j < input.rows - 2; j++) {
            for (int i = 2; i < input.cols - 2; i++) {
                output.at<float>(j, i) /= lambda_max;
            }
        }

        cv::imshow("input", input);
        cv::imshow("output", output);
        if (cv::waitKey(30) > 0) {
            break;
        }
    }
    return 0;
}
